@@ -39,6 +39,9 @@ L<pmarkdown> to learn how to set a mode.
 Note that all options are applied I<on top> of the selected mode. Even if the
 options are passed before the mode, the mode will not override the options.
 
+Note also that only the modes are tested as coherent selves. Setting individual
+options on top of a given mode might result in inconsistent behavior.
+
 =head1 OPTIONS
 
 =cut
@@ -58,6 +61,8 @@ my %validation;
 our @valid_modes = (qw(default cmark github markdown));
 my %valid_modes = map { $_ => 1 } @valid_modes;
 
+my $err_str;
+
 sub set_options {
   my ($this, $dest, @options) = @_;
   # We don’t put the options into a hash, to preserve the order in which they
@@ -69,7 +74,7 @@ sub set_options {
     } else {
       carp "Unknown option ignored: ${k}" unless exists $validation{$k};
       my $validated_value = $validation{$k}($v);
-      croak "Invalid value for option '${k}': ${ERRNO}" unless defined $validated_value;
+      croak "Invalid value for option '${k}': ${err_str}" unless defined $validated_value;
       $this->{$dest}{$k} = $validated_value;
     }
   }
@@ -87,7 +92,7 @@ sub validate_options {
     } else {
       die "Unknown option: ${k}\n" unless exists $validation{$k};
       my $validated = $validation{$k}($v);
-      die "Invalid value for option '${k}': ${ERRNO}\n" unless defined $validated;
+      die "Invalid value for option '${k}': ${err_str}\n" unless defined $validated;
     }
   }
   return;
@@ -136,9 +141,9 @@ sub _make_option {
 
 sub _boolean {
   return sub {
-    return 0 if $_[0] eq 'false' || $_[0] eq '0';
+    return 0 if $_[0] eq 'false' || $_[0] eq '' || $_[0] eq '0';
     return 1 if $_[0] eq 'true' || $_[0] eq '1';
-    $ERRNO = 'must be a boolean value (0 or 1)';
+    $err_str = 'must be a boolean value (0 or 1)';
     return;
   };
 }
@@ -147,7 +152,7 @@ sub _enum {
   my @valid = @_;
   return sub {
     return $_[0] if any { $_ eq $_[0] } @valid;
-    $ERRNO = "must be one of '".join("', '", @valid)."'";
+    $err_str = "must be one of '".join("', '", @valid)."'";
     return;
   };
 }
@@ -156,7 +161,7 @@ sub _regex {
   return sub {
     my $re = eval { qr/$_[0]/ };
     return $re if defined $re;
-    $ERRNO = 'cannot be parsed as a Perl regex ($@)';
+    $err_str = 'cannot be parsed as a Perl regex ($@)';
     return;
   };
 }
@@ -173,9 +178,42 @@ sub _word_list {
 
 =head2 Options controlling which top-level blocks are used
 
+=head3 B<parse_file_metadata> I<(enum, default: yaml)>
+
+This option controls whether the parser accepts optional metadata at the
+beginning of the file. Currently, it does nothing with these metadata, even when
+they are accepted. In the future they might be used to build complete HTML file
+instead of just fragment.
+
+The possible values are:
+
+=over 4
+
+=item B<none>
+
+No file metadata is accepted, any structure in the document must be in Markdown.
+
+=item B<yaml> I<(default)>
+
+The parser accepts a YAML table as the very beginning of the document. This
+table must start with a line containing just C<---> and ends with another line
+containing C<---> or C<...>. The content between these two markers must be valid
+a valid, non-empty YAML sequence and it cannot contain any empty line.
+
+=back
+
+=cut
+
+_make_option(
+  parse_file_metadata => 'yaml',
+  _enum(qw(none yaml)),
+  cmark => 'none',
+  github => 'none',
+  markdown => 'none',);
+
 =head3 B<use_fenced_code_blocks> I<(boolean, default: true)>
 
-This options controls whether fenced code blocks are recognised in the document
+This option controls whether fenced code blocks are recognised in the document
 structure.
 
 =cut
@@ -449,6 +487,18 @@ _make_option(
 
 =pod
 
+=head3 B<render_naked_paragraphs> I<(boolean, default: false)>
+
+When this is set to true, the C<E<lt>pE<gt>> tag is always skipped around a
+paragraph. This is mostly meant to render short amount of text as pure Markdown
+inline content, without a surrounding block structure.
+
+=cut
+
+_make_option(render_naked_paragraphs => 0, _boolean);
+
+=pod
+
 =head2 Options controlling which inline elements are used
 
 =head3 B<use_extended_autolinks> I<(boolean, default: true)>
@@ -598,6 +648,19 @@ _make_option(
 
 =pod
 
+=head3 B<two_spaces_hard_line_breaks> I<(boolean, default: false)>
+
+Controls if a hard line break can be generated by ending a line with two spaces.
+
+=cut
+
+_make_option(
+  two_spaces_hard_line_breaks => 0,
+  _boolean,
+  (markdown => 1, cmark => 1, github => 1));
+
+=pod
+
 =head2 Options controlling the rendering of inline elements
 
 =head3 B<html_escaped_characters> I<(character_class)>
@@ -612,7 +675,7 @@ C<&>, C<E<lt>>, and C<E<gt>>.
 sub _escaped_characters {
   return sub {
     return $_[0] if $_[0] =~ m/^["'&<>]*$/;
-    $ERRNO = "must only contains the following characters: \", ', &, <, and >";
+    $err_str = "must only contains the following characters: \", ', &, <, and >";
     return;
   };
 }
